# idedither - generate combined image from dither
# Author : Robert Greimel (greimel@ing.iac.es)
# Version: 06. Sep. 2001
##############################################################################
procedure idedither(input,output)

string	input	{prompt="List of dither images"}
string	output	{prompt="Name of output file"}
string	match	{prompt="Method used to match images (wcs|manual|pick1)"}
bool	subsky	{prompt="Subtract sky"}
bool	savesky	{prompt="save sky image"}
string	outsky	{prompt="name of output sky image"}
string	insky	{prompt="name of input sky image"}
string	scale	{prompt="Scale images (none|mode|median|mean|<keyword>)"}
bool	check	{prompt="Check range"}
bool	verbose	{prompt="Verbose"}
string	tmpdir	{prompt="Temporary directory for file conversion"}
# pick1 options
bool	view	{prompt="View centroiding for pick1"}
string	correct	{prompt="Method used to correct pointing (none|relative)"}
# imcentroid parameters
int	bigbox	{prompt="Size of the coarse centering box"}

string *list1	{prompt="Ignore this parameter(list1)"}
string *list2	{prompt="Ignore this parameter(list2)"}

begin
  string	ini,ino,isi,iso
  string	logf,fname,diffile,skysub,key,offset,junk,dispimg
  struct	cmd
  file		tmpl,inli,deli,sky,tf1,tf2,tcent
  real		x,y,xr,yr,ref_ra,ref_dec,tel_ra,tel_dec,xoff,yoff,imscale
  real		xmax,xmin,ymax,ymin
  real		xs,ys,xd,yd,sign
  int		wcs,nsc
  int		x1,x2,y1,y2
  bool		first,usrsky

  # scale in arcsec/pixel
  imscale = ingrid_ql.scale

  # remove any temporary files that could have been left around by a previous
  # aborted run
  delete("tmp$idedither*",yes,verify=no)

  # copy input arguments to force definition at the beginning of the script
  ini = input
  ino = output
  isi = insky
  iso = outsky

  # decide if to show output
  if ( verbose )
    logf = "STDOUT"
  else
    logf = ""

  # expand output image name.
  tmpl = mktemp("tmp$idedither")
  sections(ino,option="fullname",> tmpl)
  list1 = tmpl
  nsc = fscan(list1, fname)
  delete(tmpl,yes,verify=no)

  # check if output image exists
  imgaccess(fname, verbose=no)
  if ( imgaccess.exists )
    {
    beep
    print("")
    print("ERROR: operation would override output file "//fname)
    print("idedither aborted")
    print("")
    beep
    bye
    }
 
  # check if input sky image exists. This is not needed in wcs mode.
  usrsky = no
  if ( match != "wcs" )
    {
    imgaccess(isi, verbose=no)
    if ( imgaccess.exists )
      {
      usrsky = yes
      # get temporary file name
      dispimg = mktemp (tmpdir//"idedither")
      print("INFO: input sky image found.")
      }
    else
      {
      print("WARNING: No input sky image found. Displaying raw image.")
      }
    }

  # create unique files for expanding image names, etc.
  tmpl = mktemp("tmp$idedither")
  inli = mktemp("tmp$idedither")
  deli = mktemp("tmp$idedither")
  if ( match != "wcs" )
    {
    tf1   = mktemp("tmp$idedither")
    tf2   = mktemp("tmp$idedither")
    tcent = mktemp("tmp$idedither")
    }

  # expand input image names.
  sections(ini,option="fullname",> tmpl)
 
  # make sure we have some input files ... bail out if we don't
  first = yes
  list1 = tmpl
  while (fscan(list1, fname) != EOF)
    {
    imgaccess(fname, verbose=no)
    if ( imgaccess.exists ) first = no
    }

  if ( first )
    {
    beep
    print("")
    print("ERROR: no input files found.")
    print("idedither aborted")
    print("")
    beep
    bye
    }

  # Work through list of input files and find minimum and maximum shifts
  # based on coordinate information as well as the frame size (from the
  # first image)
  if ( check && match != "wcs" )
    {
    if ( verbose ) print("finding minimum and maximum shifts ...")
    first = yes
    list1 = tmpl
    while (fscan(list1, fname) != EOF)
      {
      # check file subtraction and format
      icheckfile(fname,verbose=no)
      sign = 1
      if ( icheckfile.format == "UDAS" ) sign = -1

      # find size of the frame for first (reference) image
      if ( first )
        {
        ifileinfo(input=fname,verbose=no)
        xmin = 1
        xmax = ifileinfo.naxis1
        ymin = 1
        ymax = ifileinfo.naxis2
        }

      # find position telescope was pointing at
      # RA is in hours
      # DEC is in degree
      # XAPNOM and XAPOFF is in degree
      # YAPNOM and YAPOFF is in degree
      imgets (image=fname//"[0]", param="RA")
      tel_ra = real(imgets.value)
      imgets (image=fname//"[0]", param="DEC")
      tel_dec = real(imgets.value)
      imgets (image=fname//"[0]", param="XAPNOM")
      xoff = real(imgets.value)
      imgets (image=fname//"[0]", param="YAPNOM")
      yoff = real(imgets.value)
      imgets (image=fname//"[0]", param="XAPOFF")
      xoff = xoff + real(imgets.value)
      imgets (image=fname//"[0]", param="YAPOFF")
      yoff = yoff + real(imgets.value)
      tel_dec = tel_dec + yoff
      # conversion factor degrees to radian: PI/180 = 0.017453293
      # conversion factor degrees to hours: 1/15
      tel_ra = tel_ra + xoff/15./cos(tel_dec*0.017453293)

      if ( first )
        {
        # store position telescope was pointing at as reference
        ref_ra = tel_ra
        ref_dec = tel_dec
        first = no
        }
      else
        {
# check if RA and DEC are half of what they should be ...
 if ( abs(ref_ra/tel_ra - 2.) < 0.02 && abs(ref_dec/tel_dec - 2.) < 0.02)
   {
   tel_ra = tel_ra*2.
   tel_dec = tel_dec*2.
   print("WARNING - RA/DEC for file "//fname//" are wrong.")
   }
# or of course it could be the other way round ...
 if ( abs(tel_ra/ref_ra - 2.) < 0.02 && abs(tel_dec/ref_dec - 2.) < 0.02)
   {
   ref_ra = ref_ra*2.
   ref_dec = ref_dec*2.
   print("WARNING - RA/DEC for reference file "//fname//" are wrong.")
   }
   
        # conversion factor degrees to radian: PI/180 = 0.017453293
        # conversion factor arcsec to hour   : 1 hour = 54000 arcsec
        # conversion factor arcsec to degree : 1 deg  = 3600 arcsec
        x = -sign*54000.*cos(ref_dec*0.017453293)*(tel_ra - ref_ra)/imscale
        y = sign*3600.*(tel_dec - ref_dec)/imscale
        # check limits
#print("x="//x//" y="//y)
#print(xmin//" "//xmax//"   "//ymin//" "//ymax)
        if ( x >= 0 && x > xmin ) xmin = x
        if ( x < 0  && 1024+x < xmax ) xmax = 1024+x
        if ( y >= 0 && y > ymin ) ymin = y
        if ( y < 0  && 1024+y < ymax ) ymax = 1024+y
        }
      }

    # print information about common area and check that there is some
    print ( "xmin = "//xmin )
    print ( "xmax = "//xmax )
    print ( "ymin = "//ymin )
    print ( "ymax = "//ymax )
    if ( xmin >= xmax || ymin >= ymax )
      {
      beep
      print("")
      print("ERROR: no common area among input files.")
      print('*****  Please re-run idedither with match="wcs"')
      print("idedither aborted")
      print("")
      beep
      bye
      }
    }

  # Work through list of input files and check if they are already pre read
  # subtracted images containing a wcs
  first = yes
  list1 = tmpl
  while (fscan(list1, fname) != EOF)
    {
    # get temporary file name
    diffile = mktemp (tmpdir//"idedither")

    # check file subtraction and format
    icheckfile(fname,verbose=no)
    sign = 1
    if ( icheckfile.format == "UDAS" ) sign = -1

    # check if data file needs to be post-pre subtracted
    if ( ! icheckfile.subtract )
      {
      # calculate post-pre image
      print ("calculating post-pre for image "//fname//" ...")
      iframediff (input = fname,
                  output = diffile,
                  wcs = yes,
                  verbose = no,
                  info = no )
      }
    else
      {
      imcopy (input = fname,
              output = diffile,
              verbose = verbose)
      }

    # check if we should subtract a user defined sky image
    if ( usrsky )
      {
      print ("subtracting input sky image "//isi//" from image "//fname//" ...")
      isubtract(diffile,isi,dispimg)
      }
    else
      {
      dispimg=diffile
      }

    print(osfn(diffile), >> inli)
    print(osfn(diffile), >> deli)

    # check if we need to get the offset manually
    # if we do, we need to display the frame, read the image cursor,
    # centroid and remember the coordinates for input to imcombine
    if ( match == "manual" )
      {
      display(dispimg,1)
      if ( first )
        print " *** please mark one star in the image with 'm'"
      else
        print " *** please mark the same star in this image with 'm'"

      key = ""
      while ( key != "m" )
        {
        imarkstar(input=diffile,
                  center="centroid",
                  verbose=yes)
        key = imarkstar.key

        if ( key == "m" )
          {
          if (first)
            {
            xr = imarkstar.x
            yr = imarkstar.y
            }
          print((xr-imarkstar.x)//" "//(yr-imarkstar.y), >> tf2)
          break
          }
        else
          beep
        }

      }

    # check if we only need to get the first offset manually
    # if we do, we need to display the frame, read the image cursor,
    # centroid and remember the coordinates for input to imcombine.
    # For all the other images we need to calculate the shift from the offset
    # and then centroid for better accuracy.
    if ( match == "pick1" )
      {
      # find position telescope was pointing at
      # RA is in hours
      # DEC is in degree
      # XAPNOM and XAPOFF is in degree
      # YAPNOM and YAPOFF is in degree
      imgets (image=diffile, param="RA")
      tel_ra = real(imgets.value)
      imgets (image=diffile, param="DEC")
      tel_dec = real(imgets.value)
      imgets (image=diffile, param="XAPNOM")
      xoff = real(imgets.value)
      imgets (image=diffile, param="YAPNOM")
      yoff = real(imgets.value)
      imgets (image=diffile, param="XAPOFF")
      xoff = xoff + real(imgets.value)
      imgets (image=diffile, param="YAPOFF")
      yoff = yoff + real(imgets.value)
      tel_dec = tel_dec + yoff
      # conversion factor degrees to radian: PI/180 = 0.017453293
      # conversion factor degrees to hours: 1/15
      tel_ra = tel_ra + xoff/15./cos(tel_dec*0.017453293)

      if ( first )
        {
        # store position telescope was pointing at as reference
        ref_ra = tel_ra
        ref_dec = tel_dec

        # set smart track error budget to zero
        xd = 0
        yd = 0

        # display image
        display(dispimg,1)
        # mark area for star to be picked
        if ( check )
          {
          x = xmin + (xmax - xmin) / 2.
          y = ymin + (ymax - ymin) / 2.
          print(int(x)//" "//int(y)) | tvmark (frame=1,\
                coords="STDIN",\
                mark = "rectangle",\
                lengths = int(xmax - xmin)//" "//(ymax-ymin)/(xmax-xmin),\
                color = 205,\
                interactive = no )
          }
        # print usage information
        print " *** please mark one star in the image with 'm'"
        if ( check ) print " *** within the green rectangle (common area)"

        key = ""
        while ( key != "m" )
          {
          imarkstar(input=diffile,
                    center="centroid",
                    verbose=yes)

          key = imarkstar.key
          if ( key == "m" )
            {
            xr = imarkstar.x
            yr = imarkstar.y
            print("0 0", >> tf2)
            break
            }
          else if ( key == "q" )
            {
            beep
            print("bailing out ...")
            beep
            bye
            }
          else
            beep
          }
        }
      else
        {
# check if RA and DEC are half of what they should be ...
 if ( abs(ref_ra/tel_ra - 2.) < 0.02 && abs(ref_dec/tel_dec - 2.) < 0.02)
   {
   tel_ra = tel_ra*2.
   tel_dec = tel_dec*2.
   }
# or of course it could be the other way round ...
 if ( abs(tel_ra/ref_ra - 2.) < 0.02 && abs(tel_dec/ref_dec - 2.) < 0.02)
   {
   ref_ra = ref_ra*2.
   ref_dec = ref_dec*2.
   }

        # conversion factor degrees to radian: PI/180 = 0.017453293
        # conversion factor arcsec to hour   : 1 hour = 54000 arcsec
        # conversion factor arcsec to degree : 1 deg  = 3600 arcsec
        x = xr + sign*54000.*cos(ref_dec*0.017453293)*(tel_ra - ref_ra)/imscale
        y = yr - sign*3600.*(tel_dec - ref_dec)/imscale

        # round to three decimal places
        xs = real (nint (x * 1000.)) / 1000.
        ys = real (nint (y * 1000.)) / 1000.

        # now correct for last offset
        if ( correct == "relative" )
          {
          xs = xs + xd
          ys = ys + yd
          }

        print("Auto       : x="//xs//" y="//ys)
        delete(tf1,yes,verify=no, >>& "/dev/null")
        print(xs//" "//ys, > tf1)

        # show centroiding image if requested
        if ( view )
          {
          x1=int(xs-bigbox/2.)
          x2=int(xs+bigbox/2.)
          y1=int(ys-bigbox/2.)
          y2=int(ys+bigbox/2.)
          display(dispimg//"["//x1//":"//x2//","//y1//":"//y2//"]",1,
                  fill=yes, >>&"/dev/null")
          }

        imcentroid(input=dispimg,
                   reference="",
                   coords=tf1,
                   shifts="",
                   boxsize=7,
                   bigbox=bigbox,
                   negative=no,
                   background=INDEF,
                   lower=INDEF,
                   upper=INDEF,
                   niterate=3,
                   tolerance=0,
                   verbose=no, > tcent)

        # read centroiding information
        list2 = tcent
        nsc = fscan (list2)
        nsc = fscan (list2, junk, x, junk, y)
        list2 = ""
        delete(tcent,yes,verify=no)

        print("Centroid   : x="//x//" y="//y)
        print((xr-x)//" "//(yr-y), >> tf2)

        # correct for relative position error
        if ( correct == "relative" )
          {
          xd = xd+x-xs
          yd = yd+y-ys
          print("Abs. Error : x="//xd//" y="//yd)
          }

        }

      }

    if ( usrsky )
      {
      imdelete(dispimg,yes,verify=no)
      }
    first = no
    }

  if ( subsky )
    {
    # calculate sky = median of unshifted images
    print "calculating sky ...."
    sky = mktemp("tmp$idedithersky")
    imcombine (\
          input = "@"//osfn(inli),
         output = sky,
        rejmask = "",
         plfile = "",
          sigma = "",
        logfile = logf,
        combine = "median",
#         reject = "sigclip",
         reject = "none",
        project = no,
        outtype = "real",
        offsets = "",
       masktype = "none",
      maskvalue = 0.,
          blank = 1.,
#          scale = "none",
          scale = scale,
           zero = "none",
         weight = "none",
        statsec = "",
        expname = "EXPTIM",
     lthreshold = INDEF,
     hthreshold = INDEF,
           nlow = 1,
          nhigh = 1,
          nkeep = 1,
          mclip = yes,
         lsigma = 3.,
         hsigma = 3.,
        rdnoise = "0.",
           gain = "1.",
         snoise = "0.",
       sigscale = 0.1,
          pclip = -0.5,
           grow = 0.)

    # get temporary file name
    skysub = mktemp (tmpdir//"idedither")

    # Work through list of images and do sky subtraction
    print "calculating image-sky ...."
    list1 = inli
    while (fscan(list1, fname) != EOF)
      {

      imrename (oldnames = fname,
                newnames = skysub,
                verbose = verbose)

      imarith (operand1 = skysub,
               op = "-",
               operand2 = sky,
               result = fname,
               title = "",
               divzero = 0.,
               hparams = "",
               pixtype = "",
               calctype = "",
               verbose = verbose,
               noact = no)
 
      imdelete(skysub,yes,verify=no)
      }

    if ( savesky )
      {
      imrename(sky,iso,verbose=no)
      }
    else
      {
      imdelete(sky,yes,verify=no)
      }
    }

  # define offset parameter
  if ( match == "wcs" )
    offset = "wcs"
  else
    offset = tf2

  # unshift and coadd images
  print "coadding images ...."
  imcombine (\
        input = "@"//osfn(inli),
       output = ino,
      rejmask = "",
       plfile = "",
        sigma = "",
      logfile = logf,
      combine = "average",
       reject = "sigclip",
      project = no,
      outtype = "real",
      offsets = offset,
     masktype = "goodvalue",
    maskvalue = 0.,
        blank = 1.,
#        scale = "exposure",
        scale = "none",
         zero = scale,
       weight = "none",
      statsec = "[100:200,100:200]",
      expname = "EXPTIM",
   lthreshold = INDEF,
   hthreshold = INDEF,
         nlow = 1,
        nhigh = 1,
        nkeep = 1,
        mclip = yes,
       lsigma = 3.,
       hsigma = 3.,
      rdnoise = "0.",
         gain = "1.",
       snoise = "0.",
     sigscale = 0.1,
        pclip = -0.5,
         grow = 0.)

  # remove bad pixel mask (BPM) header keyword from combined image. The BPM
  # has done it's job, is incorrect for the combined image but can cause
  # problems if the image is used for further processing.
  hedit(images = ino,
    fields = "BPM", value = "", add = no, delete = yes,
    verify = no, show = no, update = yes)

  # delete temporary files
  imdelete("@"//deli,yes,verify=no)
  delete(tmpl,yes,verify=no)
  delete(inli,yes,verify=no)
  delete(deli,yes,verify=no)
  if ( match == "man" )
    {
    delete(tf1,yes,verify=no)
    delete(tf2,yes,verify=no)
    }

  # clear lists
  list1 = ""
  list2 = ""

end
