# iframediff - calculates post-pre integration image
# Author     : Robert Greimel (greimel@ing.iac.es)
# Last Change: 07. Jul. 2001
##############################################################################
procedure iframediff(input,output)

file	input		{prompt="Input image(s)"}
file	output		{prompt="Output image(s)"}
bool	subdark		{prompt="Subtract Dark ?"}
file	darkimg		{prompt="Dark Image"}
bool	flatfield	{prompt="Flatfield images ?"}
file	flatimg		{prompt="flat field image"}
bool	subsky		{prompt="Subtract Sky ?"}
file	skyimg		{prompt="sky image"}
bool	wcs		{prompt="Set WCS ?"}
bool	verbose		{prompt="Verbose ?"}
bool	info		{prompt="Information messages ?"}
string	tmpdir		{prompt="temporary directory for file conversion"}

string	*list1		{prompt="Ignore this parameter(list1)"}
string	*list2		{prompt="Ignore this parameter(list2)"}

begin

  string	imgname,outname,title,stmp1,stmp2,stmp3,stmp4,stmp5,bpm
  string	btmp,inli,outli,fname
  file		askyimg,aflatimg,adarkimg,s1tmp,s2tmp,s3tmp
  bool		s1del,s2del,s3del,err
  int		i,xmax,ymax,ci,co

  #copy input arguments to force definition at the beginning of the script
  imgname=input
  outname=output
  if ( subdark ) adarkimg = darkimg
  if ( flatfield ) aflatimg = flatimg
  if ( subsky ) askyimg = skyimg

  # dark image is supposed to be post-pre subtracted
  if ( subdark )
    {
    icheckfile(input=adarkimg,verbose=no)
    if ( ! icheckfile.subtract )
      {
      beep
      error(1,"dark image "// adarkimg // " is not post-pre subtracted.")
      }
    }

  # flat field image is supposed to be post-pre subtracted
  if ( flatfield )
    {
    icheckfile(input=aflatimg,verbose=no)
    if ( ! icheckfile.subtract )
      {
      beep
      error(1,"flat field image "// aflatimg // " is not post-pre subtracted.")
      }
    }

  # sky image is supposed to be post-pre subtracted
  if ( subsky )
    {
    icheckfile(input=askyimg,verbose=no)
    if ( ! icheckfile.subtract )
      {
      beep
      error(1,"sky image "// askyimg // " is not post-pre subtracted.")
      }
    }

  # check verbosity setting
  if (verbose) info = yes

  # load package wide parameters
  bpm = ingrid_ql.bpm

  # setup variables
  # stage 1 ... post-pre
  # stage 2 ... dark subtraction
  # stage 3 ... flatfield
  # stage 4 ... subtract sky
  # stage 5 ... find woorld coordinate system
  s1tmp = ""
  s1del = no
  s2tmp = ""
  s2del = no
  s3tmp = ""
  s3del = no

  # generate unique temporary file base name
  btmp = mktemp(tmpdir//"iframdiff")

  # expand input image names.
  inli = btmp//"-inli"
  sections(imgname,option="fullname",> inli)
 
  # count number of input image files
  err = no
  ci = 0
  list1 = inli
  while (fscan(list1, fname) != EOF)
    {
    imgaccess(fname, verbose=no)
    if ( imgaccess.exists )
      {
      ci = ci+1
      }
    else
      {
      print("ERROR - input file "//fname//" does not exist")
      err=yes
      }
    }
  list1 = ""

  # expand input output image names.
  outli = btmp//"-outli"
  sections(outname,option="fullname",> outli)
 
  # count number of output files
  co = 0
  list2 = outli
  while (fscan(list2, fname) != EOF)
    {
    imgaccess(fname, verbose=no)
    if ( imgaccess.exists )
      {
      print("ERROR - output file "//fname//" does already exist")
      err=yes
      }
    else
      {
      co = co+1
      }
    }
  list2 = ""

  # check if an error in the input or output file lists was reported
  if ( err )
    {
    delete(files=inli,go_ahead=yes,verify=no,default_acti=yes,
           allversions=yes,subfiles=yes)
    delete(files=outli,go_ahead=yes,verify=no,default_acti=yes,
           allversions=yes,subfiles=yes)
    print("Errors detected in input and/or output file lists - bailing out")
    bye
    }

  # check that the number of input files is equal to the number of output files
  if ( ci != co )
    {
    delete(files=inli,go_ahead=yes,verify=no,default_acti=yes,
           allversions=yes,subfiles=yes)
    delete(files=outli,go_ahead=yes,verify=no,default_acti=yes,
           allversions=yes,subfiles=yes)
    beep
    error(2,"number of input ("//ci//") and output ("//co//") files differ")
    }

  ##########
  # loop over input file list, writing files to the output file name
  ##########
  list1 = inli
  list2 = outli
  while (fscan(list1, fname) != EOF)
    {
    imgname = fname
    if (fscan(list2, fname) == EOF)
      {
      beep
      error(2,"unexpected end of output file name list")
      }
    outname = fname

    # find subtraction status and format of image
    icheckfile(input=imgname,verbose=no)

    if ( icheckfile.subtract )
      {
      s1tmp = imgname
      }
    else
      {
      # find unique temporary file name for intermediate file
      if ( ! subdark && ! flatfield && ! subsky )
        {
        s1tmp = outname
        }
      else
        {
        s1tmp = btmp//"-s1tmp"
        s1del = yes
        }

      #####
      # calculate post-pre image
      #####
      if ( info ) print "subtracting pre read from post read ..."
      if ( icheckfile.format == "ImpB" )
        {
        # get title of image
        imgets (image=imgname, param="i_title")
        title = imgets.value

        #subtract pre read from post read
        imarith (operand1=imgname//"[*,*,2]", \
      	       op="-", \
      	       operand2=imgname//"[*,*,1]", \
      	       result=s1tmp, \
      	       title="Post-Pre of "//title, \
      	       divzero=0., \
      	       hparams="", \
      	       pixtype="1", \
      	       calctype="", \
      	       verbose=verbose, \
      	       noact=no)

        # counteract imarith problem
        # although only a 2 dimensional section is passed to imarith, imarith
        # sees the 3 dimensional information in the header and feels compelled
        # to add 3 dimensional WCS information to the 2D output it produces ...
        hedit(images=s1tmp,fields="WCSDIM",value=2,add+,del-,ver-,show-,upd+)
        }
      if ( icheckfile.format == "UDAS" )
        {
        # get title of image
        imgets (image=imgname//"[0]", param="i_title")
        title = imgets.value

        if ( icheckfile.nwindows == 0 )
          {
          # There are no windows, so just subtract pre read from post read
          imarith (operand1=imgname//"[2]", \
      	         op="-", \
      	         operand2=imgname//"[1]", \
      	         result=s1tmp, \
      	         title="Post-Pre of "//title, \
      	         divzero=0., \
      	         hparams="EXPTIME", \
      	         pixtype="1", \
      	         calctype="", \
      	         verbose=verbose, \
      	         noact=no)
          }
        else
          {
          # There is one or more windows. imcombine is used to reconstruct
          # them into a frame of original size. The pixel offsets are the
          # lower left window corners.
          # stmp1 holds the image[extension] for the pre read
          # stmp2 holds the image[extension] for the post read
          # stmp3 holds the offsets
          # stmp3 is the rebuild pre read image
          # stmp4 is the rebuild post read image
          xmax = 0
          ymax = 0
          stmp1 = btmp//"-stmp1"
          stmp2 = btmp//"-stmp2"
          stmp3 = btmp//"-stmp3"
          stmp4 = btmp//"-stmp4"
          stmp5 = btmp//"-stmp5"
          print ("# Absolute",> stmp3)

          # find offsets for windows.
          for (i=1; i<= icheckfile.nwindows; i+=1)
            {
            # write image extensions to pre and post read lists
            print(imgname//"["//i//"]", >> stmp1)
            print(imgname//"["//(i+icheckfile.nwindows)//"]", >> stmp2)

            # find window location information
            iwininfo(input=imgname,window=i,verbose=no)

            print(iwininfo.xstart//" "//iwininfo.ystart, >> stmp3)
            if ( iwininfo.xend > xmax ) xmax = iwininfo.xend
            if ( iwininfo.yend > ymax ) ymax = iwininfo.yend
            }

          # check if frame extends to maximum detector size.
          # If not use a trick to get it to full size.
          if ( xmax < 1024 || ymax < 1024 )
            {
            print(imgname//"[1][1:2,1:2]", >> stmp1)
            print(imgname//"[1][1:2,1:2]", >> stmp2)
            print("1024 1024", >> stmp3)
            }

          # combine images to form pre read image
          imcombine (\
                input = "@"//stmp1,
               output = stmp4,
              rejmask = "",
               plfile = "",
                sigma = "",
              logfile = "",
              combine = "average",
               reject = "none",
              project = no,
              outtype = "real",
              offsets = stmp3,
             masktype = "none",
                blank = 0.,
                scale = "none",
                 zero = "none",
               weight = "none",
           lthreshold = INDEF,
           hthreshold = INDEF)

          # combine images to form post read image
          imcombine (\
                input = "@"//stmp2,
               output = stmp5,
              rejmask = "",
               plfile = "",
                sigma = "",
              logfile = "",
              combine = "average",
               reject = "none",
              project = no,
              outtype = "real",
              offsets = stmp3,
             masktype = "none",
                blank = 0.,
                scale = "none",
                 zero = "none",
               weight = "none",
           lthreshold = INDEF,
           hthreshold = INDEF)

          # subtract the reconstructed pre read from the post read
          imarith (operand1=stmp5, \
      	         op="-", \
      	         operand2=stmp4, \
      	         result=s1tmp, \
      	         title="Post-Pre of "//title, \
      	         divzero=0., \
      	         hparams="EXPTIME", \
      	         pixtype="1", \
      	         calctype="", \
      	         verbose=verbose, \
      	         noact=no)

          # delete temporary files
          imdelete(images=stmp1, go_ahead=yes, verify=no, >>& "dev$null")
          imdelete(images=stmp2, go_ahead=yes, verify=no, >>& "dev$null")
          imdelete(images=stmp3, go_ahead=yes, verify=no, >>& "dev$null")
          imdelete(images=stmp4, go_ahead=yes, verify=no, >>& "dev$null")
          imdelete(images=stmp5, go_ahead=yes, verify=no, >>& "dev$null")
          }
        # counteract imarith problem
        # although the resulting file has no image extensions, EXTEND=T as
        # it was copied from the operand header
        hedit(images=s1tmp,fields="EXTEND",value="F",add+,del-,ver-,show-,upd+)
        }

      # add header item to indicate post-pre was done
      hedit(images=s1tmp,fields="IQL-PP",value="T",add+,del-,ver-,show-,upd+)
      }

    # dark subtract post-pre image
    if ( subdark )
      {
      # find unique temporary file name for intermediate file
      if (  ! flatfield && ! subsky )
        {
        s2tmp = outname
        }
      else
        {
        s2tmp = btmp//"-s2tmp"
        s2del = yes
        }

      if ( info ) print "subtracting dark ..."
      imarith (operand1 = s1tmp,
               op = "-",
               operand2 = adarkimg,
               result = s2tmp,
               title = "",
               divzero = 0.,
               hparams = "",
               pixtype = "",
               calctype = "",
               verbose = verbose,
               noact = no)

      # add header item to indicate dark subtraction was done
      hedit(images=s1tmp,fields="IQL-DARK",value=adarkimg,add+,del-,ver-,show-,upd+)
      }
    else
      {
      s2tmp = s1tmp
      }

    # flatfield post-pre image
    if ( flatfield )
      {
      # find unique temporary file name for intermediate file
      if (  ! subsky )
        {
        s3tmp = outname
        }
      else
        {
        s3tmp = btmp//"-s3tmp"
        s3del = yes
        }

      # divide by flatfield
      if ( info ) print "flatfielding ..."
      imarith (operand1 = s2tmp,
               op = "/",
               operand2 = aflatimg,
               result = s3tmp,
               title = "",
               divzero = 0.,
               hparams = "",
               pixtype = "",
               calctype = "",
               verbose = verbose,
               noact = no)

      # add header item to indicate flatfielding was done
      hedit(images=s1tmp,fields="IQL-FLAT",value=aflatimg,add+,del-,ver-,show-,upd+)
      }
    else
      {
      s3tmp = s2tmp
      }

    # sky subtract post-pre image
    if ( subsky )
      {
      if ( info ) print "subtracting sky ..."
      imarith (operand1 = s3tmp,
               op = "-",
               operand2 = askyimg,
               result = outname,
               title = "",
               divzero = 0.,
               hparams = "",
               pixtype = "",
               calctype = "",
               verbose = verbose,
               noact = no)

      # add header item to indicate sky subtraction was done
      hedit(images=s1tmp,fields="IQL-SKY",value=askyimg,add+,del-,ver-,show-,upd+)
      }

    # find sky WCS if asked for
    if ( wcs )
      {
      isetwcs ( outname, verbose=verbose )
      }

    # add header item specifying bad pixel mask
    if ( bpm != "none" )
      {
      hedit(images=s1tmp,fields="BPM",value=bpm,add+,del-,ver-,show-,upd+)
      }

    # remove temporary files if created
    if ( s1del )
      imdelete(images=s1tmp, go_ahead=yes, verify=no, >>& "dev$null")
    if ( s2del )
      imdelete(images=s2tmp, go_ahead=yes, verify=no, >>& "dev$null")
    if ( s3del )
      imdelete(images=s3tmp, go_ahead=yes, verify=no, >>& "dev$null")

    #end of loop over images
    }

  # remove (global) temporary files
  delete(files=inli,go_ahead=yes,verify=no,default_acti=yes,
         allversions=yes,subfiles=yes)
  delete(files=outli,go_ahead=yes,verify=no,default_acti=yes,
         allversions=yes,subfiles=yes)

  # reset list parameters
  list1 = ""
  list2 = ""

end
