# isectionstat - find statistics for a data section
# Author : Robert Greimel (greimel@ing.iac.es)
# Version: 30. Mar. 2001
##############################################################################
procedure isectionstat(input)

string	input			{prompt="input file"}
string	outtab			{prompt="Name of output table"}

string	field = "midpt"		{prompt="statistical quantity to calculate"}
real	value = ""		{prompt="returned value"}
real	stddev = ""		{prompt="returned value"}
string	section = "[*,*]"	{prompt="Selected image section"}
int	xdiv			{prompt="number of divisions in x"}
int	ydiv			{prompt="number of divisions in y"}

real    clip = 3		{prompt="clipping factor in sigma"}
string	center = "midpt"	{prompt="Central statistical measure",
				    enum="mean|midpt|mode"}

bool	plot = yes		{prompt="plot the result ?"}
bool	verbose			{prompt="verbose"}

string *list1			{prompt="Ignore this parameter(list1)"}
string *list2			{prompt="Ignore this parameter(list2)"}

begin
  string	ini
  string	instr,sfname,stmp1,stmp2,trim,stats1,stats2
  string	tmptab,name,filsec
  file		tmpi
  real		etime,m,s,rtmp1,rtmp2,low,high,dark,darkerr
  int		ip,xs,xe,xdp,ys,ye,ydp
  bool		first

  # copy input arguments to force definition at the beginning of the script
  ini = input

  # setup of variables
  first = yes

  # check if output table already exists
  if ( outtab != "" && access(outtab) )
    {
    beep
    print("")
    print("ERROR: operation would override output table "//outtab)
    print("isectionstat aborted")
    beep
    bye
    }

  # create unique filenames for temporary files
  tmpi = mktemp("tmp$isectionstat")

  # create temporary file to hold list of image sections to find
  # statistics for
  filsec = mktemp ("tmp$isectionstat")
  stats1 = mktemp ("tmp$isectionstat")
  stats2 = mktemp ("tmp$isectionstat")

  # create temporary table names
  tmptab = mktemp ("tmp$isectionstat")//".tab"

  # check file subtraction and format
  icheckfile(ini,verbose=no)

  # check if data file needs to be post-pre subtracted
  if ( icheckfile.subtract )
    {
    sfname = ini
    }
  else
    {
    # calculate post-pre image
    if ( verbose ) print ("calculating post-pre for image "//ini//" ...")
    iframediff (input = ini,
                output = tmpi,
                subdark = no,
                flatfield = no,
                subsky = no,
                wcs = no,
                verbose = no,
                info = no )
    sfname = tmpi
    }

  if ( first )
    {
    # get size of useful area from image header if the section was not
    # specified directly
    ip = stridx("[",section)
    if ( ip == 0 )
      {
      imgets (image=sfname, param=section)
      trim = imgets.value
      }
    else
      {
      trim = section
      }

    # extract section bounding values
    # first, split the x and y sections
    # then expand * if necessary
    # last, assign the section start and end values
    ip = stridx(",", trim)
    stmp1 = substr(trim, 2, ip-1)
    stmp2 = substr(trim, ip+1, strlen(trim)-1)

    ip = stridx(":", stmp1)
    if ( ip == 0 )
      {
      xs = 1
      imgets (image=sfname, param="i_naxis1")
      xe = int(imgets.value)
      }
    else
      {
      xs = int(substr(stmp1,1,ip-1))
      xe = int(substr(stmp1,ip+1,strlen(stmp1)))
      }
    xdp = int( (xe-xs+1) / xdiv )
    if ( verbose ) {print ( "x step size : "//xdp//" pixels" )}

    ip = stridx(":", stmp2)
    if ( ip == 0 )
      {
      ys = 1
      imgets (image=sfname, param="i_naxis2")
      ye = int(imgets.value)
      }
    else
      {
      ys = int(substr(stmp2,1,ip-1))
      ye = int(substr(stmp2,ip+1,strlen(stmp2)))
      }
    ydp = int( (ye-ys+1) / ydiv )
    if ( verbose ) {print ( "y step size : "//ydp//" pixels" )}

    first = no
    }

  # loop over x divisions
  for (i=1; i<=xdiv; i+=1)
    {
    stmp2 = sfname//"["//(xs+(i-1)*xdp)//":"//(xs+i*xdp-1)//","

    # loop over y divisions
    for (j=1; j<=ydiv; j+=1)
      {
      print (stmp2//(ys+(j-1)*ydp)//":"//(ys+j*ydp-1)//"]", >> filsec)
      }
    }

  # calculate image statistics for all sections
  if ( verbose ) {print ("calculating statistics ...")}
  imstatistics ("@"//filsec,
                fields="midpt,stddev",
                lower=INDEF,
                upper=INDEF,
                binwidth=0.1,
                format=no, > stats1)

  # re-calculate statistics with sigma clipping
  if (verbose )
    {print ("recalculating statistics with "//clip//" sigma clipping ...")}
  list1 = filsec
  list2 = stats1
  while ( fscan(list1, name) == 1 )
    {
    if (fscan (list2, rtmp1, rtmp2) != 2) goto err
    low = rtmp1 - clip*rtmp2
    high = rtmp1 + clip*rtmp2
    imstatistics (name, \
                  fields = field, \
                  lower = low, \
                  upper = high, \
                  binwidth = 0.1, \
                  format = no, >> stats2)
    }

err:
  # read statistics into table
  tcreate (table = tmptab, \
           cdfile = "ingrid_ql$isectionstat.cd", \
           datafile = stats2, \
           uparfile = "", \
           nskip = 0, \
           nlines = 0, \
           nrows = 0, \
           hist = no, \
           extrapar = 0, \
           tbltype = "default", \
           extracol = 0 )

  # repeate stringent sigma clipping on final table n times
  low = INDEF
  high = INDEF
  for (i=1; i<=1; i+=1)
    {
    # calculate average mean and stdev
    tstat (intable = tmptab, \
           column = "mean", \
           outtable = "", \
           lowlim = low,\
           highlim = high )

    # find limits for recalculation of final statistics
    if ( center == "mean" )
      {
      low = tstat.mean - 1*tstat.stddev
      high = tstat.mean + 1*tstat.stddev
      }
    else
      {
      low = tstat.median - 1*tstat.stddev
      high = tstat.median + 1*tstat.stddev
      }
    }

  # last recalculation with more relaxed sigma clipping again.
  if ( center == "mean" )
    {
    low = tstat.mean - 2*tstat.stddev
    high = tstat.mean + 2*tstat.stddev
    }
  else
    {
    low = tstat.median - 2*tstat.stddev
    high = tstat.median + 2*tstat.stddev
    }

  tstat (intable = tmptab, \
         column = "mean", \
         outtable = "", \
         lowlim = low,\
         highlim = high )
  m = tstat.mean
  s = tstat.stddev

  # plot results if requested
  if ( plot )
    {
    axispar.xlabel = "Median ADU"
    axispar.ylabel = "Number"
    axispar.title = "Histogram for Median of "//ini//section//" "//str(m)
    thistogram (tmptab,"STDOUT","mean") | sgraph (crvstyle="pseudohist")
    }

  value=m
  stddev=s

  # print verbose information
  if ( verbose ) print(ini//" : "//m//" "//s)

  # now delete temporary files
  delete(tmptab,yes,verify=no)
  delete(stats1,yes,verify=no)
  delete(stats2,yes,verify=no)
  delete(filsec,yes,verify=no)

  # write results to output table if specified
  if ( outtab != "" )
    {
    }

  # delete temporary images and files
  imdelete(images=tmpi, go_ahead=yes, verify=no, >>& "/dev/null")

  # clear list variables
  list1 = ""
  list2 = ""

end
