# setwcs - calculates WCS for INGRID image from header information
# Author     : Robert Greimel (greimel@ing.iac.es)
# Last Change: 06. Sep. 2001
##############################################################################
procedure isetwcs(input)

file	input		{prompt="Input image(s)"}
string	platescale	{prompt="Platescale or Keyword"}
bool	verbose		{prompt="Verbose ?"}

begin

  string	imgname,detector
  real		pixsize,pscale
  real		xmag,ymag,tel_ra,tel_dec,xoff,yoff,sign

  # copy input arguments
  imgname=input
  pscale = real("0"//platescale)

  # find detector from header information
  imgets (image=imgname, param="DETECTOR")
  detector = imgets.value

  # check file subtraction and format
  icheckfile(imgname,verbose=no)
  sign = 1
  if ( icheckfile.format == "UDAS" ) sign = -1

  # read and calculate plate scale if necessary
  if ( pscale == 0. )
    {
    imgets (image=imgname, param=platescale)
    pscale = real(imgets.value)
    # check if detector is known and set necessary values
    # pixsize ... pixel size in [m]
    if ( detector == "HAWAII-1" )
      {
      pixsize=0.0000185   # 18.5 mu-m
      }
    else
      {
      print ("Detector : "//detector)
      print ("Sorry - this detector is unknown")
      return
      }
    # calculate scale
    ymag = sign * pscale * pixsize * 3600.
    xmag = -ymag
    }
  else
    {
    ymag = sign * pscale
    xmag = -ymag
    }

  # find position telescope was pointing at
  imgets (image=imgname, param="RA")
  tel_ra = real(imgets.value)
  imgets (image=imgname, param="DEC")
  tel_dec = real(imgets.value)
  imgets (image=imgname, param="XAPOFF")
  xoff = real(imgets.value)
  imgets (image=imgname, param="YAPOFF")
  yoff = real(imgets.value)
  tel_ra = tel_ra + xoff/15.
  tel_dec = tel_dec + yoff
  if ( verbose )
    {
    print ("RA ="//tel_ra)
    print ("Dec="//tel_dec)
    }

  # set wcs
  # xref,yref ... pixel coordinates of sky reference point (default=center)
  # xmag,ymag ... scale factors in "/pixel
  # xrotaion,yrotation ... rotation angels (counter-clockwise)
  # lngref,latref ... RA,Dec for sky reference point
  # lngunits,latunits ... units [hours|degrees|radians]
  # projection ... sky projection geometry [tan|arc|sin|lin|ait|car|csc|gls|mer|
  #                                         mol|par|pco|qsc|stg|tsc|zea]
  # coosystem ... celectial coordinate system [icrs|j2000|b1950|...]
  ccsetwcs (images=imgname, \
	  database="", \
	  solutions="", \
	  xref=INDEF, \
	  yref=INDEF, \
	  xmag=xmag, \
	  ymag=ymag, \
	  xrotation=INDEF, \
	  yrotation=INDEF, \
	  lngref=tel_ra, \
	  latref=tel_dec, \
#	  lngunits="degrees", \
#	  latunits="degrees", \
	  lngunits=INDEF, \
	  latunits=INDEF, \
	  transpose=no, \
	  projection="tan", \
	  coosystem="j2000", \
	  update=yes, \
	  pixsystem="logical", \
	  verbose=verbose )

end
