# istarfocus - wrapper for nmisc/starfocus
# Author : Robert Greimel  greimel@ing.iac.es,
# Version: 09. Apr. 2001
##############################################################################
procedure istarfocus (images)

string	images	{prompt="List of focusrun images"}
string	skyimg	{prompt="sky image"}
string	focus	{prompt="focus header keyword"}
string  tmpdir  {prompt="temporary directory for file conversion"}

string *list1   {prompt="Ignore this parameter(list1)"}

begin
  string	ini,infoc
  string	fname,diffile
  file		tmpl,inli,deli,sky,tf1

  # setup
  sky = ""

  # copy input arguments to force definition at the beginning of the script
  ini = images
  infoc = focus

  # create a unique file for expanding image names given.
  tmpl = mktemp("tmp$istarf")
  inli = mktemp("tmp$istarf")
  deli = mktemp("tmp$istarf")

  if ( skyimg != "" )
    {
    # check if we can access the sky image
    imgaccess( skyimg, verbose=no )
    if ( !imgaccess.exists)
      {
      print("cannot access the specified sky image : "//skyimg)
      print("Please check file name or reset the skyimg parameter.")
      print("Use 'unlearn istarfocus' to reset all istarfocus parameters")
      beep
      bye
      }

    # check file subtraction and format
    icheckfile(skyimg,verbose=no)

    # check if sky image needs to be post-pre subtracted
    if ( ! icheckfile.subtract )
      {
      sky = mktemp (tmpdir//"/istarf")

      # calculate post-pre image
      print ("calculating post-pre image for sky ...")
      iframediff (input=skyimg, \
                  output=sky)
      print(osfn(sky), >> deli)
      }
    }

  # expand image name.
  sections(ini,option="fullname",> tmpl)

  # Work through list of input files and check if they are already pre read
  # subtracted images
  list1 = tmpl
  while (fscan(list1, fname) != EOF)
    {
    # check file subtraction and format
    icheckfile(fname,verbose=no)

    # check if data file needs to be post-pre subtracted
    if ( ! icheckfile.subtract )
      {
      diffile = mktemp (tmpdir//"/istarf")

      # calculate post-pre image
      print ("calculating post-pre image ...")
      iframediff (input=fname, \
                  output=diffile)
      print(osfn(diffile), >> inli)
      print(osfn(diffile), >> deli)
      }
    else
      {
      if ( sky != "" )
        {
        diffile = mktemp (tmpdir//"/istarf")
        imcopy (input = fname,
                output = diffile,
                verbose = no)
        print(osfn(diffile), >> inli)
        print(osfn(diffile), >> deli)
        }
      else
        {
        print(osfn(fname), >> inli)
        }
      }

    if ( sky != "" )
      {
      tf1 = mktemp (tmpdir//"/istarf")
      imrename (oldnames = diffile,
                newnames = tf1,
                verbose = no)

      imarith (operand1 = tf1,
               op = "-",
               operand2 = sky,
               result = diffile,
               title = "",
               divzero = 0.,
               hparams = "",
               pixtype = "",
               calctype = "",
               verbose = no,
               noact = no)

      imdelete(tf1,yes,verify=no)

      }

    }

  # call starfocus
  starfocus(\
       images = "@"//inli,
        focus = infoc,
        fstep = "",
   nexposures = "1",
         step = "30",
    direction = "-line",
          gap = "end",
       coords = "mark1",
          wcs = "logical",
      display = yes,
        frame = 1,
        level = 0.5,
         size = "FWHM",
         beta = INDEF,
        scale = 1.,
       radius = 5.,
      sbuffer = 5.,
       swidth = 5.,
   saturation = INDEF,
   ignore_sat = no,
   iterations = 2,
      xcenter = INDEF,
      ycenter = INDEF,
      logfile = "logfile",
     imagecur = "",
     graphcur = "")

  # delete temporary images and files
  imdelete("@"//deli,yes,verify=no)
  delete(tmpl,yes,verify=no)
  delete(inli,yes,verify=no)
  delete(deli,yes,verify=no)

end
