# istatistics - generate statistics for post and pre read images
# Author : Robert Greimel (greimel@ing.iac.es)
# Version: 17. Dec. 2000
##############################################################################
procedure istatistics(input)

string	input		{prompt="List of images"}
string	sections	{prompt="Section to be processed"}
# imstat parameters
string	fields		{prompt="Fields to be printed"}
real	lower		{prompt="Lower cutoff for pixel values"}
real	upper		{prompt="Upper cutoff for pixel values"}
real	binwidth	{prompt="Bin width of histogram in sigma"}
bool	format		{prompt="Format output and print column labels?"}

string	*list1	{prompt="Ignore this parameter(list1)"}

begin
  bool		bpre,bpost,bdata
  int		cpos
  string	ini
  string	fname,fsec,fsecp,tsec,ttsec
  file		tmpl,inli,tmpp,tf

  # initialize variables
  bpre = no
  bpost = no
  bdata = no

  # copy input arguments to force definition at the beginning of the script
  ini = input
  tsec = sections

  # check sections string for what user wants statistics calculated.
  while (strlen(tsec) > 0)
    {
    cpos = stridx(",",tsec)
    if (cpos == 0)
      {
      ttsec = tsec
      tsec = ""
      }
    else
      {
      ttsec = substr(tsec,1,cpos-1)
      tsec = substr(tsec,cpos+1,strlen(tsec))
      }
    # check for known sections
    if (ttsec == "pre") {bpre = yes}
    else if (ttsec == "post") {bpost = yes}
    else if (ttsec == "data") {bdata = yes}
    else {print("Warning - Unknown section : "//ttsec)}
    }

  # check that at least one section was specified
  if ( !(bpre || bpost || bdata) )
    {
    print("No section specified - aborting")
    print("")
    beep
    bye
    }

  # create temporary file names
  tmpl = mktemp("tmp$istat")
  tmpp = mktemp("tmp$istat")
  inli = mktemp("tmp$istat")

  # expand image name.
  sections(ini,option="fullname",> tmpl)

  # Work through list of input files and check if they are already pre read
  # subtracted images
  list1 = tmpl
  while (fscan(list1, fname) != EOF)
    {
    # check file subtraction and format
    icheckfile(fname,verbose=no)

    # split filename into name + section information
    cpos = stridx("[",fname)
    if (cpos > 0)
      {
      # copy the section string apart from the closing ]
      # generate section argument for post-pre processed files (includes ])
      # and remove the section from the filename
      fsec = substr(fname,cpos,strlen(fname)-1)
      fsecp = fsec//"]"
      fname = substr(fname,1,cpos-1)
      }
    else
      {
      # set default section if none was given
      if ( icheckfile.subtract )
        {
        fsec = ""
        fsecp = ""
        }
      else
        {
        fsec = "[*,*"
        fsecp = ""
        }
      }

    # find statistics for post and pre or post-pre
    if ( icheckfile.subtract )
      {
      if ( bpre ) {print("Warning - file "//fname//" is missing pre section")}
      if ( bpost ) {print("Warning - file "//fname//" is missing post section")}
      if ( bdata ) {print(fname//fsecp, >> inli)}
      }
    else
      {
      if ( icheckfile.format == "ImpB" )
        {
        if ( bpre ) {print(fname//fsec//",1]", >> inli)}
        if ( bpost ) {print(fname//fsec//",2]", >> inli)}
        }
      if ( icheckfile.format == "UDAS" )
        {
        if ( bpre ) {print(fname//"[1]"//fsec//"]", >> inli)}
        if ( bpost ) {print(fname//"[2]"//fsec//"]", >> inli)}
        }
      # file is not post-pre subtracted, so if data is asked for we need
      # to do this. This creates a temporary file that we need to delete
      # at the end of the task.
      if ( bdata )
        {
        tf = mktemp("tmp$istat")
        print(tf, >> tmpp)
        print(tf//fsecp, >> inli)
        # calculate post-pre image
        iframediff (\
          fname,
          output=tf,
          subdark=no,
          flatfield=no,
          subsky=no,
          wcs=no,
          verbose=no,
          info=no)
        }
      }
    }

  # calculate image statistics
  if ( access(inli) )
    {
    imstatistics (\
         images = "@"//inli,
         fields = fields,
          lower = lower,
          upper = upper,
       binwidth = binwidth,
         format = format)
    }

  # delete temporary images and files
  if ( access(tmpp) )
    {
    imdelete("@"//tmpp,yes,verify=no)
    delete(tmpp,yes,verify=no)
    }
  delete(tmpl,yes,verify=no)
  if (access(inli)) {delete(inli,yes,verify=no)}

end
