# itiming - find timing offsets from linearity data
# Author : Robert Greimel (greimel@ing.iac.es)
# Version: 01. Apr. 2001
##############################################################################
procedure itiming (lintab1,lintab2)

string	lintab1		{prompt="Name of first linearity table"}
string	lintab2		{prompt="Name of second linearity table"}

real	adumin		{prompt="minimum ADU for fit"}
real	adumax		{prompt="maximum ADU for fit"}

string	function	{prompt="Interpolation function"}

string	*list1

begin
  string	inlt1,inlt2,tmptab,tmptab2,tmptab3,key,cmd
  string	tfil1,fittab1,fittab2,rangetab1,rangetab2,rangetab3
  int		gwcs,order,i
  real		amin,amax,gx,gy,a1,b1,a2,b2

  # copy input parameters
  inlt1 = lintab1
  inlt2 = lintab2
  amin = adumin
  amax = adumax

  # defines
  order = 4

  # create temporary table
  tmptab = mktemp ("tmp$itabplot")
  tmptab2 = tmptab//"_t2.tab"
  tmptab3 = tmptab//"_t3.tab"
  fittab1 = tmptab//"_f1.tab"
  fittab2 = tmptab//"_f2.tab"
  rangetab1 = tmptab//"_r1.tab"
  rangetab2 = tmptab//"_r2.tab"
  rangetab3 = tmptab//"_r3.tab"
  tmptab = tmptab//".tab"

  # temporary files
  tfil1 = mktemp ("tmp$itabplot")

  # copy flux, median and exposure time to new table
  tcopy(intable = inlt1//"[c:median1,flux1,exp1]", 
        outtable = tmptab2, 
        verbose = no)
  tchcol(table = tmptab2, 
         oldname = "median1", 
         newname = "m1", 
         newfmt = "", 
         newunits = "", 
         verbose = no)
  tchcol(table = tmptab2, 
         oldname = "flux1", 
         newname = "f1", 
         newfmt = "", 
         newunits = "", 
         verbose = no)
  tchcol(table = tmptab2, 
         oldname = "exp1", 
         newname = "e1", 
         newfmt = "", 
         newunits = "", 
         verbose = no)
  tsort (table = tmptab2,
         columns = "m1",
	 ascend = yes,
	 casesens = no)

  tcopy(intable = inlt2//"[c:median1,flux1,exp1]", 
        outtable = tmptab3, 
        verbose = no)
  tchcol(table = tmptab3, 
         oldname = "median1", 
         newname = "m2", 
         newfmt = "", 
         newunits = "", 
         verbose = no)
  tchcol(table = tmptab3, 
         oldname = "flux1", 
         newname = "f2", 
         newfmt = "", 
         newunits = "", 
         verbose = no)
  tchcol(table = tmptab3, 
         oldname = "exp1", 
         newname = "e2", 
         newfmt = "", 
         newunits = "", 
         verbose = no)
  tsort (table = tmptab3,
         columns = "m2",
	 ascend = yes,
	 casesens = no)


  tmerge(intable = tmptab2//","//tmptab3,
         outtable = tmptab, 
         option = "merge", 
         allcols = yes, 
         tbltype = "default", 
         allrows = 100, 
         extracol = 0)
  # delete what is no longer needed
  delete(tmptab2,yes,verify=no)
  delete(tmptab3,yes,verify=no)

  ########################################################################
  # find common range
  tselect(tmptab,tmptab2,"m1 >= "//amin//" && m1 <= "//amax)
  tselect(tmptab,tmptab3,"m2 >= "//amin//" && m2 <= "//amax)

  tstat (intable = tmptab2, 
         column = "m1", 
         outtable = "", 
         lowlim = INDEF,
         highlim = INDEF )
  if ( tstat.nrows == 0 )
    {
    print ("ERROR: No data between "//amin//" and "//amax)
    beep
    bye
    }
  if ( tstat.vmin > amin ) amin=tstat.vmin
  if ( tstat.vmax < amax ) amax=tstat.vmax
  delete(tmptab2,yes,verify=no)

  tstat (intable = tmptab3, 
         column = "m2", 
         outtable = "", 
         lowlim = INDEF,
         highlim = INDEF )
  if ( tstat.nrows == 0 )
    {
    print ("ERROR: No common ADU range in data.")
    beep
    bye
    }
  if ( tstat.vmin > amin ) amin=tstat.vmin
  if ( tstat.vmax < amax ) amax=tstat.vmax
  delete(tmptab3,yes,verify=no)

  print("Common range: "//amin//" to "//amax)

  ########################################################################
  # resample first flux curve
  tcopy(tmptab//"[c:m1,f1,e1]",tmptab2,verbose=no)
  trebin(intable = tmptab2,
         outtable = fittab1,
         column = "m1",
         start = amin,
         end = amax,
         step = (amax-amin)/100,
         xtable = "",
         function = function,
         extrapolate = no,
         value = INDEF,
         padvalue = INDEF,
         verbose = no)
  delete(tmptab2,yes,verify=no)

  ########################################################################
  # resample second flux curve
  tcopy(tmptab//"[c:m2,f2,e2]",tmptab2,verbose=no)
  trebin(intable = tmptab2,
         outtable = fittab2,
         column = "m2",
         start = amin,
         end = amax,
         step = (amax-amin)/100,
         xtable = "",
         function = function,
         extrapolate = no,
         value = INDEF,
         padvalue = INDEF,
         verbose = no)
  delete(tmptab2,yes,verify=no)

  ########################################################################
  # assumtion is that for long exposure times the difference in counts
  # will be independent of any timing error.
  # To bring both flux curves to the same scale we therefor devide by
  # the smallest available value in the fit.
  tstat (intable = fittab1, 
         column = "f1", 
         outtable = "", 
         lowlim = INDEF,
         highlim = INDEF )
  tcalc (table = fittab1,
         outcol = "s1",
         equals = "f1/"//tstat.vmin)

  tstat (intable = fittab2, 
         column = "f2", 
         outtable = "", 
         lowlim = INDEF,
         highlim = INDEF )
  tcalc (table = fittab2,
         outcol = "s2",
         equals = "f2/"//tstat.vmin)

  ########################################################################
  # trying to calculate timing error ...
  tmerge(intable = fittab1//"[c:m1,f1,e1],"//fittab2//"[c:m2,f2,e2]", 
         outtable = tmptab2, 
         option = "merge", 
         allcols = yes, 
         tbltype = "default", 
         allrows = 200, 
         extracol = 0)

  # Method 1
  tstat (intable = tmptab2, 
         column = "e1", 
         outtable = "", 
         lowlim = INDEF,
         highlim = INDEF )
  gx = tstat.vmax
  tstat (intable = tmptab2, 
         column = "e2", 
         outtable = "", 
         lowlim = INDEF,
         highlim = INDEF )
  gy = tstat.vmax
  tcalc (table = tmptab2,
         outcol = "terror",
         equals = "( e1*"//gy//"-"//gx//"*e2 ) / ( "//gx//"+e2-e1-"//gy//" )")
  tcalc (table = tmptab2,
         outcol = "tratio",
         equals = "e1/e2")
  tdump (table = tmptab2,
         cdfile = "",
	 pfile = "",
	 datafile = "STDOUT",
	 columns = "m1 tratio",
	 rows = "-",
	 pwidth = 80) | polyfit (input = "STDIN",
	 order = 2,
	 weighting = "uniform",
         verbose = yes,
         listdata = no)

  # Method 2
  tselect(tmptab//"[c:m1,e1]",rangetab1,"m1 >= "//amin//" && m1 <= "//amax)
  tselect(tmptab//"[c:m2,e2]",rangetab2,"m2 >= "//amin//" && m2 <= "//amax)
  tdump (table = rangetab1,
         cdfile = "",
	 pfile = "",
	 datafile = "STDOUT",
	 columns = "e1 m1",
	 rows = "-",
	 pwidth = 80) | polyfit (input = "STDIN",
	 order = 1,
	 weighting = "uniform",
         verbose = yes,
         listdata = no, >>tfil1)
  list1 = tfil1
  i = fscan(list1,a1,b1)
  delete(tfil1,yes,verify=no)

  tdump (table = rangetab2,
         cdfile = "",
	 pfile = "",
	 datafile = "STDOUT",
	 columns = "e2 m2",
	 rows = "-",
	 pwidth = 80) | polyfit (input = "STDIN",
	 order = 1,
	 weighting = "uniform",
         verbose = yes,
         listdata = no, >>tfil1)
  list1 = tfil1
  i = fscan(list1,a2,b2)
  delete(tfil1,yes,verify=no)

  print("Flux Ratio: "//(b2/b1))

  tcopy(intable = rangetab2,
        outtable = rangetab3, 
        verbose = no)
  tcalc (table = rangetab3,
         outcol = "m2",
         equals = "m2/"//b2//"*"//b1)

  tdump (table = rangetab1,
         cdfile = "",
	 pfile = "",
	 datafile = "STDOUT",
	 columns = "m1 e1",
	 rows = "-",
	 pwidth = 80) | polyfit (input = "STDIN",
	 order = 2,
	 weighting = "uniform",
         verbose = yes,
         listdata = no)
  tdump (table = rangetab3,
         cdfile = "",
	 pfile = "",
	 datafile = "STDOUT",
	 columns = "m2 e2",
	 rows = "-",
	 pwidth = 80) | polyfit (input = "STDIN",
	 order = 2,
	 weighting = "uniform",
         verbose = yes,
         listdata = no)

  ########################################################################
  # Plot the results
  #
  # some general plot parameter settings
  pltpar.szmarker = 0.01
  # set key to generate initial plot
  key = "a"
  while ( key != "q" )
    {
    switch ( key )
      {
      ###
      # plot Flux vs. ADU
      ###
      case 'a':
        {
        # plot both flux curves
        itabplot(itable1 = tmptab,
                 col1 = "m1 f1",
                 itable2 = tmptab,
                 col2 = "m2 f2",
                 title = "INGRID Linearity",
                 xlabel = "Median [ADU]",
                 ylabel = "Flux [ADU/s]",
                 point = yes,
                 line = yes,
                 pavg = no,
                 xerr = no,
                 yerr = no)
        }
      ###
      # plot Flux vs. ADU for first data and fit
      ###
      case 'b':
        {
        # plot first flux curve and fit
        itabplot(itable1 = tmptab,
                 col1 = "m1 f1",
                 itable2 = fittab1,
                 col2 = "m1 f1",
                 title = "INGRID Linearity",
                 xlabel = "Median [ADU]",
                 ylabel = "Flux [ADU/s]",
                 point = yes,
                 line = yes,
                 pavg = no,
                 xerr = no,
                 yerr = no)
        }
      ###
      # plot Flux vs. ADU for second data and fit
      ###
      case 'c':
        {
        # plot second flux curve and fit
        itabplot(itable1 = tmptab,
                 col1 = "m2 f2",
                 itable2 = fittab2,
                 col2 = "m2 f2",
                 title = "INGRID Linearity",
                 xlabel = "Median [ADU]",
                 ylabel = "Flux [ADU/s]",
                 point = yes,
                 line = yes,
                 pavg = no,
                 xerr = no,
                 yerr = no)
        }
      ###
      # plot normalized Flux vs. common ADU
      ###
      case 'd':
        {
        # plot both flux curves
        itabplot(itable1 = fittab1,
                 col1 = "m1 s1",
                 itable2 = fittab2,
                 col2 = "m2 s2",
                 title = "INGRID Linearity",
                 xlabel = "Median [ADU]",
                 ylabel = "Normalized Flux [ADU/s]",
                 point = yes,
                 line = yes,
                 pavg = no,
                 xerr = no,
                 yerr = no)
        }
      ###
      # plot timing error vs. common ADU
      ###
      case 'f':
        {
        # plot both flux curves
        itabplot(itable1 = tmptab2//"[r:row=1:99]",
                 col1 = "m1 terror",
                 itable2 = tmptab2//"[r:row=1:99]",
                 col2 = "m2 terror",
                 title = "INGRID Linearity",
                 xlabel = "Median [ADU]",
                 ylabel = "Timing Error [s]",
                 point = yes,
                 line = yes,
                 pavg = no,
                 xerr = no,
                 yerr = no)
        }
      ###
      # plot Exposure Time vs. ADU
      ###
      case 'g':
        {
        # plot both flux curves
        itabplot(itable1 = tmptab,
                 col1 = "e1 m1",
                 itable2 = tmptab,
                 col2 = "e2 m2",
                 title = "INGRID Linearity",
                 xlabel = "Exposure Time [s]",
                 ylabel = "Median [ADU]",
                 point = yes,
                 line = yes,
                 pavg = no,
                 xerr = no,
                 yerr = no)
        }
      ###
      # plot exposure time ratio vs. common ADU
      ###
      case 'h':
        {
        # plot both flux curves
        itabplot(itable1 = tmptab2,
                 col1 = "m1 tratio",
                 itable2 = tmptab2,
                 col2 = "m2 tratio",
                 title = "INGRID Linearity",
                 xlabel = "Median [ADU]",
                 ylabel = "Exposure Time Ratio",
                 point = yes,
                 line = yes,
                 pavg = no,
                 xerr = no,
                 yerr = no)
        }
      ###
      # plot ADU vs. exposure time
      ###
      case 'i':
        {
        # plot both flux curves
        itabplot(itable1 = tmptab,
                 col1 = "e1 m1",
                 itable2 = tmptab,
                 col2 = "e2 m2",
                 title = "INGRID Linearity",
                 xlabel = "Exposure Time",
                 ylabel = "ADU",
                 point = yes,
                 line = yes,
                 pavg = no,
                 xerr = no,
                 yerr = no)
        }
      ###
      # plot ADU vs. exposure time
      ###
      case 'j':
        {
        # plot both flux curves
        itabplot(itable1 = tmptab2,
                 col1 = "e1 m1",
                 itable2 = tmptab2,
                 col2 = "e2 m2",
                 title = "INGRID Linearity",
                 xlabel = "Exposure Time",
                 ylabel = "ADU",
                 point = yes,
                 line = yes,
                 pavg = no,
                 xerr = no,
                 yerr = no)
        }
      ###
      # plot ADU vs. exposure time
      ###
      case 'k':
        {
        # plot both flux curves
        itabplot(itable1 = rangetab1,
                 col1 = "e1 m1",
                 itable2 = rangetab3,
                 col2 = "e2 m2",
                 title = "INGRID Linearity",
                 xlabel = "Exposure Time",
                 ylabel = "ADU",
                 point = yes,
                 line = yes,
                 pavg = no,
                 xerr = no,
                 yerr = no)
        }
      ###
      # plot exposure time ratio vs. exposure time
      ###
      case 'l':
        {
        # plot both flux curves
        itabplot(itable1 = tmptab2,
                 col1 = "e1 tratio",
                 itable2 = tmptab2,
                 col2 = "e2 tratio",
                 title = "INGRID Linearity",
                 xlabel = "Exposure Time [s]",
                 ylabel = "Exposure Time Ratio",
                 point = yes,
                 line = yes,
                 pavg = no,
                 xerr = no,
                 yerr = no)
        }
      ###
      # quit
      ###
      case 'q':
        {
        goto ende
        }

      # end of switch
      }
    # read graphics cursor to determine what to plot next ...
    if ( fscan(gcur, gx, gy, gwcs, cmd) != EOF )
      {
      # find the key pressed
      key = substr(cmd, 1, 1)
      }
    else
      {
      # time to leave
      key = "q"
      }
  # end of while
  }

  #########################################################################
  # The End
ende:
  # remove temporary table
  delete(rangetab1,yes,verify=no)
  delete(rangetab2,yes,verify=no)
  delete(rangetab3,yes,verify=no)
  delete(fittab1,yes,verify=no)
  delete(fittab2,yes,verify=no)
  delete(tmptab,yes,verify=no)
  delete(tmptab2,yes,verify=no)

end
